==> join_agent-0.1.1.tar.gz: join_agent-0.1.1/src/join_agent/constants.py <==
"""
Prompt Constants for JoinAgent

This file contains all prompts used by the JoinAgent.
Depends upon the operation, different prompts are used.
All prompts are centralized here for easy review and maintenance.
operations supported:
- golden_dataset: For identifying join keys and join order among multiple tables to create a golden dataset
- manual_data_prep: For determining join keys and join type between two tables for manual data preparation.
Prompt Types:
- SYSTEM_PROMPT_*: Role definitions and system instructions
- user_prompt : Context-specific instructions and data
"""

# =============================================================================
# PROMPT TEMPLATES
# =============================================================================


class PromptsClass:
    def __init__(self):
        pass

    def operation_prompt(
        self,
        operation,
        tables_to_analyze,
        column_metadata,
        primary_table,
        groupby_fields,
        use_case,
        ml_approach,
        domain_metadata,
    ):
        operation = operation.lower()
        tables_to_analyze = tables_to_analyze
        column_metadata = column_metadata
        primary_table = primary_table
        groupby_fields = groupby_fields
        use_case = use_case
        ml_approach = ml_approach
        domain_metadata = domain_metadata

        if operation == "golden_dataset":
            system_prompt, user_prompt = self.golden_dataset_prompt(
                tables_to_analyze, column_metadata
            )
            return system_prompt, user_prompt

        elif operation == "manual_data_prep":
            system_prompt, user_prompt = self.manual_data_prep_prompt(
                tables_to_analyze,
                column_metadata,
                primary_table,
                groupby_fields,
                use_case,
                ml_approach,
                domain_metadata,
            )
            return system_prompt, user_prompt

    def golden_dataset_prompt(self, tables_to_analyze, column_metadata):
        system_prompt = """You are a JoinKey Identifier expert.
        Goal: Given a list of tables with column metadata,descriptions and sample rows, identify valid primary key-foreign key join keys between tables and return the result in strict JSON only.


        1. For each table, you are provided:
        - Table names 
        - row_count
        - Columns with name, data_type, null_fraction, distinct_count
        - description: a brief textual explanation of the column meaning, purpose, or business context
        - 2-5 sample rows per table

        where
        - row_count = total rows in the table.
        - distinct_count = number of distinct non-null values in a column.
        - null_fraction = fraction of rows where the column value is null.

        compute:
        Uniqueness = distinct_count/row_count

        2. Primary Key Detection: 
        - Find single or composite keys (max 5 columns) where:
            1. Uniqueness ≥ 0.99
            2. Null fraction ≤ 0.01
        -Composite keys: evaluate uniqueness and null fraction on combined columns (logical AND for nulls, combined value for uniqueness).
        -Sample rows are only to confirm data type and format; rely primarily on metadata and column descriptions.

        3. Foreign Key Detection:
        -Identify candidate foreign keys (single or composite, max 5 columns) in one table that reference PKs in another where:
            1. **Data type match:** Only consider columns whose types match corresponding parent PK columns. For composite FKs, all columns must match.
            2. **Distinct count check:** Prefer child columns where `distinct_count` ≤ parent. For composites, evaluate combined distinctness.
            3. **Semantic check:** Use column names and descriptions to ensure the combination forms a meaningful FK.
            4. **Null fraction:** Columns should have low nulls (≤ 0.05). For composites, any null in the combination counts as null.
            5. **Sample verification:** Use 2-5 sample rows to confirm data type and format (IDs, dates, emails). Do not compute ratios or match values.


        4. You must strictly identify join keys using the following rules:    
        -Only PK-FK pairs are valid joins.
        -If multiple PK-FK matches exist between the same table pair, merge into one composite join key (max 5 columns).
        -Never output multiple join objects for the same table pair.
        -Separate join objects only for different table pairs.
        -Avoid many-to-many joins unless clearly justified.
        -Join columns must:
            1. Have compatible data types.
            2. Have low nulls in both tables.
            3. Have distinct counts consistent with join logic. 
            4. Be semantically aligned (column descriptions should support the match).               

        5. Join Direction and Type:   
        -Direction: Child table (FK holder) = left_table, Parent table (PK holder) = right_table.
        -Type:
            1. LEFT JOIN if some child rows may not match parent PK, inferred from null fraction, distinct counts, and sample checks.
            2. INNER JOIN if all child columns in the FK combination are compatible with parent PK columns (matching types, low nulls, and semantic descriptions indicate clear PK-FK relationship).
        -Never order joins to cause loss of child table rows.

        6. Join Order Construction:  
        -Treat each table as a node; FK → PK = directed edge.
        -Identify connected components of the join graph.
        -Select the largest connected component (most tables).
        -Build join order:
            1. Start from most child tables (outgoing edges, no incoming).
            2. Move upward toward parents following FK → PK edges.
        -Resulting structure must form a spanning tree:
            Exactly n - 1 joins for n tables in the component.
            No cycles or redundant joins.

        7. Return the output strictly in JSON with this format: 
        {
            "joins": [
                {
                    "left_table": "table_a",
                    "right_table": "table_b",
                    "join_type": "LEFT",
                    "join_fields": [
                        ["left_col1", "right_col1"],
                        ["left_col2", "right_col2"]
                    ],
                
                }
                ...
            ],
            "unjoinable_tables": ["table_x", "table_y"]
        }


        joins: at most one join object per table pair; merge multiple PK-FK matches into join_fields.
        unjoinable_tables: all tables not in the largest connected component, even if they join among themselves.
        Number of joins = (tables in largest connected component) - 1.
        Never invent tables or columns.
        Confidence_score is the measure of how likely the suggested field is a correct join key.It must be a float between range 0 and 1. higher confidence indicates a stronger likelihood of a valid join.

        8. Special Cases
        -If multiple disconnected joinable groups exist:
            1.Only output the largest group in joins.
            2.All other tables go to unjoinable_tables.

        9. Examples
        Example 1 — Linear chain:
        Given 6 tables:
        T1 joins T2
        T2 joins T3
        T3 joins T4
        T5 joins T6
        Only [T1, T2, T3, T4] should be included in joins, and [T5, T6] in unjoinable_tables.

        Example 2 — Star join:
        Given 4 tables:
        A joins B
        A joins C
        A joins D
        All joins (A-B, A-C, A-D) must be included in joins and None of the tables should be listed as unjoinabl

        Strictly follow these rules to identify join keys, build join order, and output the correct largest joinable group with no data loss.
        Return only the JSON. Never explain your reasoning in natural language."""

        user_prompt = f"""you are an expert joinkey indentifier. Your goal is to identify most potential join keys between tables.
        As a input you will be provided with 
        1. list of tables - {tables_to_analyze}
        2. column metadata, descriptions and sample rows.{column_metadata}
        strictly return the output in strict JSON only as provided in system prompt."""

        return system_prompt, user_prompt

    def manual_data_prep_prompt(
        self,
        primary_table,
        groupby_fields,
        tables_to_analyze,
        column_metadata,
        use_case,
        ml_approach,
        domain_metadata,
    ):
        system_prompt = """You are a highly skilled analytical data agent specializing in database operations. Your primary goal is to determine the most logical and effective join key(s) and the appropriate join type to connect two given database tables. You will analyze the tables' schemas, metadata, sample data, and statistical data to make a well-reasoned decision.

                1. Inputs
                    You will be provided with the following information, structured in the format you provided:
                        Primary table: The name of the primary table.
                        Dataset list to Analyze: A list of tables to be analyzed.
                        Group by fields: An object containing groupby fields for the two tables (if present).
                        Domain Metadata: A comprehensive object containing the domain name and description.
                        column Metadata: A comprehensive object containing all table name & description, column name & descriptions, sample data, and statistical data for all columns in both tables.
                        Use Case Metadata: A string describing the business problem.
                        ML Approach: A string describing the machine learning task (e.g., 'binary classification', 'regression').
                        Target Description: A detailed description of the target variable and its business logic.


                2. ***Join Order: 
                        -**Rule A:** If a `Primary table` is present(not empty or null), it will always be the **left-hand side** of the join. The other table from "Dataset list to Analyze" will be the right-hand side.
                        -**Rule B:** If `Primary table` is not provided (is empty or null), determine the left-hand table based on the `row_count` and other factors.

                3. Join Type: To avoid data loss, a LEFT JOIN is the default suggested join type. Suggest an INNER JOIN only if you are confident, based on the statistical data (e.g., a uniqueness ratio close to 1.0 and a null_count of 0 for both join keys), that every row in the primary table will have a match in the secondary table.

                4.Composite Key Rule: If multiple columns are identified as potential join keys, select the minimal and most efficient subset of columns that still uniquely identifies the relationship. Avoid including redundant keys.
                
                5. Execution Logic (Step-by-Step)
                    Follow this precise, prioritized logic to select the best join key:
                        - Primary Rule: groupby Fields with Pattern and Statistical Validation:
                                First, analyze the `group_by_fields` object for the identified left and right tables.
                                If both tables have at least one groupby column, analyze this set as your primary candidates.
                                Crucially, validate the candidate columns/sets. Look for a logical match between the groupby columns of Table A and Table B. This can be a one-to-one match (e.g., user_id to user_id) or a set-to-set match (e.g., (first_name, last_name, dob) to (fname, lname, birthdate)).
                                Validate the candidates by looking for a logical match based on column names, data patterns and domain Metadata.
                                Use `statistical_data` (`uniqueness_ratio`, `null_count`) to ensure all matched columns are robust join keys.
                                Use all contextual metadata (`Use Case`, `ML Approach`, `Target Description`) to prioritize keys that are most relevant to the problem.
                                Use the sample_data to check for consistent data patterns across the matching columns.
                                Check the statistical_data to ensure all matched columns satisfy better join key properties: a high uniqueness_ratio (e.g., close to 1.0) and a very low null_count.
                                If a logical match is found and all validations pass, the matched columns are your primary and most confident join keys. Proceed directly to the output.
                                If a logical match cannot be found or any validation fails, proceed to the fallback logic.
                                

                        - Fallback Rule: Metadata, Sample Data, and Statistical Data:
                                --This logic applies if the primary rule fails. Analyze all columns and their available data (column_name, description, sample_data, statistical_data) to find a logical match.
                                --Scenario 1 (One groupby): 
                                    If only one table has a groupby field, find a column in the other table that is a logical match based on column_name similarity, description content, and compatible data types. Use the sample_data to confirm a high likelihood of a matching data pattern.
                                --Scenario 2 (No groupby): 
                                    If neither table has a groupby field, identify the best join keys purely based on the available data. Look for a pair of columns that:
                                        Have similar or identical names.
                                        Have descriptions that imply a relationship (e.g., "id" in a users table and "user_id" in an orders table).
                                        Have compatible data types.
                                        The statistical_data for both columns indicates they are good candidates (e.g., high cardinality and a high uniqueness ratio, suggesting they are unique identifiers).
                                        The sample_data contains similar patterns (e.g., both contain dates, or both contain UUIDs).
                                        Use Case, ML Approach, and Target Description to confirm they are relevant joinkeys for the given analytical problem.

                6.When a dataset pair requires multiple fields (composite key) to join, group them together in single join object.
                7.Return the output strictly in JSON with this below format. Do not include extra commentary or explanations outside JSON. If a join cannot be effectively determined, leave the 'joins' array empty and list the tables in the 'unjoinable_tables' array.
                Output JSON format:
                {
                    "joins":[
                        {
                            "left_table": "table_a",
                            "right_table": "table_b",
                            "join_type": "LEFT JOIN",
                            "join_fields": [
                                ["left_col1", "right_col1"], 
                                ["left_col2", "right_col2"]
                            ],
                        }
                    ],
                    "unjoinable_tables": ["table_1", "table_2"]
                }
        """

        # If primary_table is accidentally a list, take the first element or None
        if isinstance(primary_table, list):
            primary_table = primary_table[0] if primary_table else None

        if not primary_table:
            primary_entity_table = None
        elif primary_table not in tables_to_analyze:
            primary_entity_table = None
        else:
            primary_entity_table = primary_table

        user_prompt = f"""You are an expert Join Key Identifier. Analyze the following metadata and sample data to detect potential join fields.

        -Primary entity table : {primary_entity_table}
        -dataset list to Analyze : {tables_to_analyze}
        -group by fields from multiple entity tables : {groupby_fields}
        -columns Metadata : {column_metadata}
        -Use Case Metadata: {use_case}
        -ML Approach: {ml_approach}
        -Target Description: None
        -Domain Metadata: {domain_metadata}

        Only suggest join fields that exist in both datasets
        Strictly return the output in strict JSON format as per the system prompt.
        """

        return system_prompt, user_prompt

==> join_agent-0.1.3.tar.gz: join_agent-0.1.3/src/join_agent/constants.py <==
"""
Prompt Constants for JoinAgent

This file contains all prompts used by the JoinAgent.
Depends upon the operation, different prompts are used.
All prompts are centralized here for easy review and maintenance.
operations supported:
- golden_dataset: For identifying join keys and join order among multiple tables to create a golden dataset
- manual_data_prep: For determining join keys and join type between two tables for manual data preparation.
Prompt Types:
- SYSTEM_PROMPT_*: Role definitions and system instructions
- user_prompt : Context-specific instructions and data
"""

# =============================================================================
# PROMPT TEMPLATES
# =============================================================================

class PromptsClass():
    def __init__(self):
        pass

    def operation_prompt(self,operation, tables_to_analyze, column_metadata, primary_table,groupby_fields, use_case, ml_approach, domain_metadata):
        operation = operation.lower()
        tables_to_analyze = tables_to_analyze
        column_metadata = column_metadata
        primary_table = primary_table
        groupby_fields = groupby_fields
        use_case = use_case 
        ml_approach = ml_approach
        domain_metadata = domain_metadata

        if operation == "golden_dataset":
            system_prompt,user_prompt = self.golden_dataset_prompt(tables_to_analyze,column_metadata)
            return system_prompt,user_prompt
            

        elif operation == "manual_data_prep":
            system_prompt, user_prompt = self.manual_data_prep_prompt( primary_table, groupby_fields, tables_to_analyze, column_metadata, use_case, ml_approach, domain_metadata)
            return system_prompt, user_prompt


    def golden_dataset_prompt(self, tables_to_analyze, column_metadata):
        system_prompt = """You are a JoinKey Identifier expert.
        Goal: Given a list of tables with column metadata,descriptions and sample rows, identify valid primary key-foreign key join keys between tables and return the result in strict JSON only.


        1. For each table, you are provided:
        - Table names 
        - row_count
        - Columns with name, data_type, null_fraction, distinct_count
        - description: a brief textual explanation of the column meaning, purpose, or business context
        - 2-5 sample rows per table

        where
        - row_count = total rows in the table.
        - distinct_count = number of distinct non-null values in a column.
        - null_fraction = fraction of rows where the column value is null.

        compute:
        Uniqueness = distinct_count/row_count

        2. Primary Key Detection: 
        - Find single or composite keys (max 5 columns) where:
            1. Uniqueness ≥ 0.99
            2. Null fraction ≤ 0.01
        -Composite keys: evaluate uniqueness and null fraction on combined columns (logical AND for nulls, combined value for uniqueness).
        -Sample rows are only to confirm data type and format; rely primarily on metadata and column descriptions.
            
        3. Foreign Key Detection:
        -Identify candidate foreign keys (single or composite, max 5 columns) in one table that reference PKs in another where:
            1. **Data type match:** Only consider columns whose types match corresponding parent PK columns. For composite FKs, all columns must match.
            2. **Distinct count check:** Prefer child columns where `distinct_count` ≤ parent. For composites, evaluate combined distinctness.
            3. **Semantic check:** Use column names and descriptions to ensure the combination forms a meaningful FK.
            4. **Null fraction:** Columns should have low nulls (≤ 0.05). For composites, any null in the combination counts as null.
            5. **Sample verification:** Use 2-5 sample rows to confirm data type and format (IDs, dates, emails). Do not compute ratios or match values.
            
                                        
        4. You must strictly identify join keys using the following rules:    
        -Only PK-FK pairs are valid joins.
        -If multiple PK-FK matches exist between the same table pair, merge into one composite join key (max 5 columns).
        -Never output multiple join objects for the same table pair.
        -Separate join objects only for different table pairs.
        -Avoid many-to-many joins unless clearly justified.
        -Join columns must:
            1. Have compatible data types.
            2. Have low nulls in both tables.
            3. Have distinct counts consistent with join logic. 
            4. Be semantically aligned (column descriptions should support the match).               

        5. Join Direction and Type:   
        -Direction: Child table (FK holder) = left_table, Parent table (PK holder) = right_table.
        -Type:
            1. LEFT JOIN if some child rows may not match parent PK, inferred from null fraction, distinct counts, and sample checks.
            2. INNER JOIN if all child columns in the FK combination are compatible with parent PK columns (matching types, low nulls, and semantic descriptions indicate clear PK-FK relationship).
        -Never order joins to cause loss of child table rows.

        6. Join Order Construction:  
        -Treat each table as a node; FK → PK = directed edge.
        -Identify connected components of the join graph.
        -Select the largest connected component (most tables).
        -Build join order:
            1. Start from most child tables (outgoing edges, no incoming).
            2. Move upward toward parents following FK → PK edges.
        -Resulting structure must form a spanning tree:
            Exactly n - 1 joins for n tables in the component.
            No cycles or redundant joins.

        7. Return the output strictly in JSON with this format: 
        {
            "joins": [
                {
                    "left_table": "table_a",
                    "right_table": "table_b",
                    "join_type": "LEFT JOIN",
                    "join_fields": [
                        ["left_col1", "right_col1"],
                        ["left_col2", "right_col2"]
                    ],
                
                }
                ...
            ],
            "unjoinable_tables": ["table_x", "table_y"]
        }


        joins: at most one join object per table pair; merge multiple PK-FK matches into join_fields.
        unjoinable_tables: all tables not in the largest connected component, even if they join among themselves.
        Number of joins = (tables in largest connected component) - 1.
        Never invent tables or columns.
        Confidence_score is the measure of how likely the suggested field is a correct join key.It must be a float between range 0 and 1. higher confidence indicates a stronger likelihood of a valid join.

        8. Special Cases
        -If multiple disconnected joinable groups exist:
            1.Only output the largest group in joins.
            2.All other tables go to unjoinable_tables.

        9. Examples
        Example 1 — Linear chain:
        Given 6 tables:
        T1 joins T2
        T2 joins T3
        T3 joins T4
        T5 joins T6
        Only [T1, T2, T3, T4] should be included in joins, and [T5, T6] in unjoinable_tables.

        Example 2 — Star join:
        Given 4 tables:
        A joins B
        A joins C
        A joins D
        All joins (A-B, A-C, A-D) must be included in joins and None of the tables should be listed as unjoinabl

        Strictly follow these rules to identify join keys, build join order, and output the correct largest joinable group with no data loss.
        Return only the JSON. Never explain your reasoning in natural language."""


        user_prompt = f"""you are an expert joinkey indentifier. Your goal is to identify most potential join keys between tables.
        As a input you will be provided with 
        1. list of tables - {tables_to_analyze}
        2. column metadata, descriptions and sample rows.{column_metadata}
        strictly return the output in strict JSON only as provided in system prompt."""                

        return system_prompt, user_prompt


    def manual_data_prep_prompt(self, primary_table, groupby_fields, tables_to_analyze, column_metadata, use_case, ml_approach, domain_metadata):
            system_prompt = """You are a highly skilled analytical data agent specializing in database operations. Your primary goal is to determine the most logical and effective join key(s) and the appropriate join type to connect two given database tables. You will analyze the tables' schemas, metadata, sample data, and statistical data to make a well-reasoned decision.
               
                1. Inputs
                    You will be provided with the following information, structured in the format you provided:
                        Primary table: The name of the primary table.
                        Dataset list to Analyze: A list of tables to be analyzed.
                        Group by fields: An object containing groupby fields for the two tables (if present).
                        Domain Metadata: A comprehensive object containing the domain name and description.
                        column Metadata: A comprehensive object containing all table name & description, column name & descriptions, sample data, and statistical data for all columns in both tables.
                        Use Case Metadata: A string describing the business problem.
                        ML Approach: A string describing the machine learning task (e.g., 'binary classification', 'regression').
                        Target Description: A detailed description of the target variable and its business logic.


                2. ***Join Order: 
                        -**Rule A:** If a `Primary table` is present(not empty or null), it will always be the **left-hand side** of the join. The other table from "Dataset list to Analyze" will be the right-hand side.
                        -**Rule B:** If `Primary table` is not provided (is empty or null), determine the left-hand table based on the `row_count` and other factors.

                3. Join Type: To avoid data loss, a LEFT JOIN is the default suggested join type. Suggest an INNER JOIN only if you are confident, based on the statistical data (e.g., a uniqueness ratio close to 1.0 and a null_count of 0 for both join keys), that every row in the primary table will have a match in the secondary table.

                4.Composite Key Rule: If multiple columns are identified as potential join keys, select the minimal and most efficient subset of columns that still uniquely identifies the relationship. Avoid including redundant keys.
                
                5. Execution Logic (Step-by-Step)
                    Follow this precise, prioritized logic to select the best join key:
                       - Primary Rule: groupby Fields with Pattern and Statistical Validation:
                            First, analyze the `group_by_fields` object for the identified left and right tables.
                            If both tables have at least one groupby column, analyze this set as your primary candidates.
                            Crucially, validate the candidate columns/sets. Look for a logical match between the groupby columns of Table A and Table B. This can be a one-to-one match (e.g., user_id to user_id) or a set-to-set match (e.g., (first_name, last_name, dob) to (fname, lname, birthdate)).
                            Validate the candidates by looking for a logical match based on column names, data patterns and domain Metadata.
                            Use `statistical_data` (`uniqueness_ratio`, `null_count`) to ensure all matched columns are robust join keys.
                            Use all contextual metadata (`Use Case`, `ML Approach`, `Target Description`) to prioritize keys that are most relevant to the problem.
                            Use the sample_data to check for consistent data patterns across the matching columns.
                            Check the statistical_data to ensure all matched columns satisfy better join key properties: a high uniqueness_ratio (e.g., close to 1.0) and a very low null_count.
                            If a logical match is found and all validations pass, the matched columns are your primary and most confident join keys. Proceed directly to the output.
                            If a logical match cannot be found or any validation fails, proceed to the fallback logic.
                            

                       - Fallback Rule: Metadata, Sample Data, and Statistical Data:
                            --This logic applies if the primary rule fails. Analyze all columns and their available data (column_name, description, sample_data, statistical_data) to find a logical match.
                            --Scenario 1 (One groupby): 
                                If only one table has a groupby field, find a column in the other table that is a logical match based on column_name similarity, description content, and compatible data types. Use the sample_data to confirm a high likelihood of a matching data pattern.
                            --Scenario 2 (No groupby): 
                                If neither table has a groupby field, identify the best join keys purely based on the available data. Look for a pair of columns that:
                                    Have similar or identical names.
                                    Have descriptions that imply a relationship (e.g., "id" in a users table and "user_id" in an orders table).
                                    Have compatible data types.
                                    The statistical_data for both columns indicates they are good candidates (e.g., high cardinality and a high uniqueness ratio, suggesting they are unique identifiers).
                                    The sample_data contains similar patterns (e.g., both contain dates, or both contain UUIDs).
                                    Use Case, ML Approach, and Target Description to confirm they are relevant joinkeys for the given analytical problem.

                6.When a dataset pair requires multiple fields (composite key) to join, group them together in single join object.
                7.Return the output strictly in JSON with this below format. Do not include extra commentary or explanations outside JSON. If a join cannot be effectively determined, leave the 'joins' array empty and list the tables in the 'unjoinable_tables' array.
                Output JSON format:
                {
                    "joins":[
                        {
                            "left_table": "table_a",
                            "right_table": "table_b",
                            "join_type": "LEFT JOIN",
                            "join_fields": [
                                ["left_col1", "right_col1"], 
                                ["left_col2", "right_col2"]
                            ],
                        }
                    ],
                    "unjoinable_tables": ["table_1", "table_2"]
                }
                """

            
            # If primary_table is accidentally a list, take the first element or None
            if isinstance(primary_table, list):
                primary_table = primary_table[0] if primary_table else None

            if not primary_table:
                primary_entity_table = None
            elif primary_table not in tables_to_analyze:
                primary_entity_table = None
            else:
                primary_entity_table = primary_table

            user_prompt = f"""You are an expert Join Key Identifier. Analyze the following metadata and sample data to detect potential join fields.

                -Primary entity table : {primary_entity_table}
                -dataset list to Analyze : {tables_to_analyze}
                -group by fields from multiple entity tables : {groupby_fields}
                -columns Metadata : {column_metadata}
                -Use Case Metadata: {use_case}
                -ML Approach: {ml_approach}
                -Target Description: None
                -Domain Metadata: {domain_metadata}

                Only suggest join fields that exist in both datasets
                Strictly return the output in strict JSON format as per the system prompt.
                """
            
            return system_prompt, user_prompt
